@@ -222,22 +222,16 @@ class SingleRoomAgent(Agent):
                 if requirement.name not in schemas_by_name:
                     installed = True
 
-                    schema_path = f".schemas/{requirement.name}.json"
+                    logger.info(f"Installing required schema {requirement.name}")
 
-                    if await self._room.storage.exists(path=schema_path):
-                        # Schema is already in the room
-                        pass
+                    if requirement.name.startswith("https://"):
+                        url = requirement.name
                     else:
-                        logger.info(f"Installing required tool {requirement.name}")
+                        url = f"{meshagent_base_url()}/schemas/{requirement.name}"
 
-                        if requirement.name.startswith("https://"):
-                            url = requirement.name
-                        else:
-                            url = f"{meshagent_base_url()}/schemas/{requirement.name}"
-
-                        await self._room.agents.make_call(
-                            url=url, name=requirement.name, arguments={}
-                        )
+                    await self._room.agents.make_call(
+                        url=url, name=requirement.name, arguments={}
+                    )
 
             else:
                 raise RoomException("unsupported requirement")
@@ -341,7 +335,7 @@ class TaskRunner(SingleRoomAgent):
         requires=None,
         supports_tools: Optional[bool] = None,
         input_schema: dict,
-        output_schema: dict,
+        output_schema: Optional[dict] = None,
         labels: Optional[list[str]] = None,
         toolkits: Optional[list[Toolkit]] = None,
     ):
