@@ -1,4 +1,4 @@
-from .agent import SingleRoomAgent, AgentChatContext
+from meshagent.agents.agent import SingleRoomAgent, AgentChatContext
 from meshagent.api.chan import Chan
 from meshagent.api import (
     RoomMessage,
@@ -10,8 +10,8 @@ from meshagent.api import (
     MeshDocument,
 )
 from meshagent.tools import Toolkit, ToolContext
-from .adapter import LLMAdapter, ToolResponseAdapter
-from meshagent.openai.tools.responses_adapter import ImageGenerationTool
+from meshagent.agents.adapter import LLMAdapter, ToolResponseAdapter
+from meshagent.openai.tools.responses_adapter import ImageGenerationTool, LocalShellTool
 import asyncio
 from typing import Optional
 import logging
@@ -22,14 +22,59 @@ from typing import Literal
 import base64
 from openai.types.responses import ResponseStreamEvent
 from asyncio import CancelledError
+from meshagent.api import RoomException
 
 from opentelemetry import trace
+import shlex
 
 tracer = trace.get_tracer("meshagent.chatbot")
 
 logger = logging.getLogger("chat")
 
 
+class ChatBotThreadLocalShellTool(LocalShellTool):
+    def __init__(self, *, thread_context: "ChatThreadContext"):
+        super().__init__()
+        self.thread_context = thread_context
+
+    async def execute_shell_command(
+        self,
+        context,
+        *,
+        command,
+        env,
+        type,
+        timeout_ms=None,
+        user=None,
+        working_directory=None,
+    ):
+        messages = None
+
+        for prop in self.thread_context.thread.root.get_children():
+            if prop.tag_name == "messages":
+                messages = prop
+                break
+
+        exec_element = messages.append_child(
+            tag_name="exec",
+            attributes={"command": shlex.join(command), "pwd": working_directory},
+        )
+
+        result = await super().execute_shell_command(
+            context,
+            command=command,
+            env=env,
+            type=type,
+            timeout_ms=timeout_ms,
+            user=user,
+            working_directory=working_directory,
+        )
+
+        exec_element.set_attribute("result", result)
+
+        return result
+
+
 class ChatBotThreadOpenAIImageGenerationTool(ImageGenerationTool):
     def __init__(
         self,
@@ -92,6 +137,7 @@ class ChatBotThreadOpenAIImageGenerationTool(ImageGenerationTool):
         for prop in self.thread_context.thread.root.get_children():
             if prop.tag_name == "messages":
                 messages = prop
+                break
 
         for child in messages.get_children():
             if child.get_attribute("id") == item_id:
@@ -141,6 +187,7 @@ class ChatBotThreadOpenAIImageGenerationTool(ImageGenerationTool):
         for prop in self.thread_context.thread.root.get_children():
             if prop.tag_name == "messages":
                 messages = prop
+                break
 
         for child in messages.get_children():
             if child.get_attribute("id") == item_id:
@@ -188,6 +235,7 @@ class ChatThreadContext:
         *,
         chat: AgentChatContext,
         thread: MeshDocument,
+        path: str,
         participants: Optional[list[RemoteParticipant]] = None,
     ):
         self.thread = thread
@@ -196,6 +244,7 @@ class ChatThreadContext:
 
         self.participants = participants
         self.chat = chat
+        self.path = path
 
 
 # todo: thread should stop when participant stops?
@@ -262,13 +311,23 @@ class ChatBot(SingleRoomAgent):
 
     async def _send_and_save_chat(
         self,
-        messages: Element,
+        thread: MeshDocument,
         path: str,
         to: RemoteParticipant,
         id: str,
         text: str,
         thread_attributes: dict,
     ):
+        messages = None
+
+        for prop in thread.root.get_children():
+            if prop.tag_name == "messages":
+                messages = prop
+                break
+
+        if messages is None:
+            raise RoomException("messages element was not found in thread document")
+
         with tracer.start_as_current_span("chatbot.thread.message") as span:
             span.set_attributes(thread_attributes)
             span.set_attribute("role", "assistant")
@@ -294,10 +353,10 @@ class ChatBot(SingleRoomAgent):
                 },
             )
 
-    async def greet(
+    async def _greet(
         self,
         *,
-        messages: Element,
+        thread: MeshDocument,
         path: str,
         chat_context: AgentChatContext,
         participant: RemoteParticipant,
@@ -308,7 +367,7 @@ class ChatBot(SingleRoomAgent):
             await self._send_and_save_chat(
                 id=str(uuid.uuid4()),
                 to=RemoteParticipant(id=participant.id),
-                messages=messages,
+                thread=thread,
                 path=path,
                 text=self._auto_greet_message,
                 thread_attributes=thread_attributes,
@@ -358,34 +417,95 @@ class ChatBot(SingleRoomAgent):
     async def close_thread(self, *, path: str):
         return await self.room.sync.close(path=path)
 
-    async def _spawn_thread(self, path: str, messages: Chan[RoomMessage]):
-        self.room.developer.log_nowait(
-            type="chatbot.thread.started", data={"path": path}
-        )
-        chat_context = await self.init_chat_context()
-        opened = False
+    async def load_thread_context(self, *, thread_context: ChatThreadContext):
+        """
+        load the thread from the thread document by inserting the current messages in the thread into the chat context
+        """
+        thread = thread_context.thread
+        chat_context = thread_context.chat
+        for prop in thread.root.get_children():
+            if prop.tag_name == "messages":
+                doc_messages = prop
 
+                for element in doc_messages.get_children():
+                    if isinstance(element, Element) and element.tag_name == "message":
+                        msg = element["text"]
+                        if element[
+                            "author_name"
+                        ] == self.room.local_participant.get_attribute("name"):
+                            chat_context.append_assistant_message(msg)
+                        else:
+                            chat_context.append_user_message(msg)
+
+                        for child in element.get_children():
+                            if child.tag_name == "file":
+                                chat_context.append_assistant_message(
+                                    f"the user attached a file with the path '{child.get_attribute('path')}'"
+                                )
+
+                break
+
+        if doc_messages is None:
+            raise Exception("thread was not properly initialized")
+
+    async def prepare_llm_context(self, *, context: ChatThreadContext):
+        """
+        called prior to sending the request to the LLM in case the agent needs to modify the context prior to sending
+        """
+        pass
+
+    async def _process_llm_events(
+        self,
+        *,
+        thread_context: ChatThreadContext,
+        llm_messages: asyncio.Queue,
+        thread_attributes: dict,
+    ):
+        thread = thread_context.thread
         doc_messages = None
-        current_file = None
-        llm_messages = Chan[ResponseStreamEvent]()
-        thread_context = None
+        for prop in thread.root.get_children():
+            if prop.tag_name == "messages":
+                doc_messages = prop
+                break
 
-        thread_attributes = None
+        if doc_messages is None:
+            raise RoomException("messages element is missing from thread document")
 
-        def done_processing_llm_events(task: asyncio.Task):
+        context_message = None
+        updates = asyncio.Queue()
+
+        # throttle updates so we don't send too many syncs over the wire at once
+        async def update_thread():
             try:
-                task.result()
-            except CancelledError:
+                changes = {}
+                while True:
+                    try:
+                        element, partial = updates.get_nowait()
+                        changes[element] = partial
+
+                    except asyncio.QueueEmpty:
+                        for e, p in changes.items():
+                            e["text"] = p
+
+                        changes.clear()
+
+                        e, p = await updates.get()
+                        changes[e] = p
+
+                        await asyncio.sleep(0.1)
+
+            except asyncio.QueueShutDown:
+                # flush any pending changes
+                for e, p in changes.items():
+                    e["text"] = p
+
+                changes.clear()
                 pass
-            except Exception as e:
-                logger.error("error sending delta", exc_info=e)
 
-        async def process_llm_events():
-            partial = ""
-            content_element = None
-            context_message = None
-
-            async for evt in llm_messages:
+        update_thread_task = asyncio.create_task(update_thread())
+        try:
+            while True:
+                evt = await llm_messages.get()
                 for participant in self._room.messaging.get_participants():
                     logger.debug(
                         f"sending event {evt.type} to {participant.get_attribute('name')}"
@@ -395,6 +515,7 @@ class ChatBot(SingleRoomAgent):
 
                 if evt.type == "response.content_part.added":
                     partial = ""
+
                     content_element = doc_messages.append_child(
                         tag_name="message",
                         attributes={
@@ -409,11 +530,11 @@ class ChatBot(SingleRoomAgent):
                     )
 
                     context_message = {"role": "assistant", "content": ""}
-                    chat_context.messages.append(context_message)
+                    thread_context.chat.messages.append(context_message)
 
                 elif evt.type == "response.output_text.delta":
                     partial += evt.delta
-                    content_element["text"] = partial
+                    updates.put_nowait((content_element, partial))
                     context_message["content"] = partial
 
                 elif evt.type == "response.output_text.done":
@@ -427,9 +548,21 @@ class ChatBot(SingleRoomAgent):
                         span.set_attribute("role", "assistant")
                         span.set_attributes(thread_attributes)
                         span.set_attributes({"text": evt.text})
+        except asyncio.QueueShutDown:
+            pass
+        finally:
+            updates.shutdown()
+            await update_thread_task
 
-        llm_task = asyncio.create_task(process_llm_events())
-        llm_task.add_done_callback(done_processing_llm_events)
+    async def _spawn_thread(self, path: str, messages: Chan[RoomMessage]):
+        logger.debug("chatbot is starting a thread", extra={"path": path})
+        chat_context = await self.init_chat_context()
+        opened = False
+
+        current_file = None
+        thread_context = None
+
+        thread_attributes = None
 
         thread = None
 
@@ -438,9 +571,9 @@ class ChatBot(SingleRoomAgent):
 
             while True:
                 while True:
-                    logger.info(f"waiting for message on thread {path}")
+                    logger.debug(f"waiting for message on thread {path}")
                     received = await messages.recv()
-                    logger.info(f"received message on thread {path}: {received.type}")
+                    logger.debug(f"received message on thread {path}: {received.type}")
 
                     chat_with_participant = None
                     for participant in self._room.messaging.get_participants():
@@ -495,97 +628,57 @@ class ChatBot(SingleRoomAgent):
 
                             thread = await self.open_thread(path=path)
 
-                            for prop in thread.root.get_children():
-                                if prop.tag_name == "messages":
-                                    doc_messages = prop
+                            thread_context = ChatThreadContext(
+                                path=path,
+                                chat=chat_context,
+                                thread=thread,
+                                participants=get_thread_participants(
+                                    room=self.room, thread=thread
+                                ),
+                            )
 
-                                    for element in doc_messages.get_children():
-                                        if isinstance(element, Element):
-                                            msg = element["text"]
-                                            if (
-                                                element["author_name"]
-                                                == self.room.local_participant.get_attribute(
-                                                    "name"
-                                                )
-                                            ):
-                                                chat_context.append_assistant_message(
-                                                    msg
-                                                )
-                                            else:
-                                                chat_context.append_user_message(msg)
-
-                                            for child in element.get_children():
-                                                if child.tag_name == "file":
-                                                    chat_context.append_assistant_message(
-                                                        f"the user attached a file with the path '{child.get_attribute('path')}'"
-                                                    )
-
-                            if doc_messages is None:
-                                raise Exception("thread was not properly initialized")
+                            await self.load_thread_context(
+                                thread_context=thread_context
+                            )
 
                     if received.type == "opened":
                         if not opened:
                             opened = True
 
-                            await self.greet(
+                            await self._greet(
                                 path=path,
                                 chat_context=chat_context,
                                 participant=chat_with_participant,
-                                messages=doc_messages,
+                                thread=thread,
                                 thread_attributes=thread_attributes,
                             )
 
                     if received.type == "chat":
                         if thread is None:
-                            self.room.developer.log_nowait(
-                                type="thread is not open", data={}
-                            )
+                            logger.info("thread is not open", extra={"path": path})
                             break
 
-                        for participant in get_thread_participants(
-                            room=self._room, thread=thread
-                        ):
-                            # TODO: async gather
-                            self._room.messaging.send_message_nowait(
-                                to=participant,
-                                type="thinking",
-                                message={"thinking": True, "path": path},
+                        logger.debug(
+                            "chatbot received a chat",
+                            extra={
+                                "context": chat_context.id,
+                                "participant_id": self.room.local_participant.id,
+                                "participant_name": self.room.local_participant.get_attribute(
+                                    "name"
+                                ),
+                                "text": received.message["text"],
+                            },
+                        )
+
+                        attachments = received.message.get("attachments", [])
+                        text = received.message["text"]
+
+                        for attachment in attachments:
+                            chat_context.append_assistant_message(
+                                message=f"the user attached a file at the path '{attachment['path']}'"
                             )
 
-                        if chat_with_participant.id == received.from_participant_id:
-                            self.room.developer.log_nowait(
-                                type="llm.message",
-                                data={
-                                    "context": chat_context.id,
-                                    "participant_id": self.room.local_participant.id,
-                                    "participant_name": self.room.local_participant.get_attribute(
-                                        "name"
-                                    ),
-                                    "message": {
-                                        "content": {
-                                            "role": "user",
-                                            "text": received.message["text"],
-                                        }
-                                    },
-                                },
-                            )
-
-                            attachments = received.message.get("attachments", [])
-                            text = received.message["text"]
-
-                            for attachment in attachments:
-                                chat_context.append_assistant_message(
-                                    message=f"the user attached a file at the path '{attachment['path']}'"
-                                )
-
-                            chat_context.append_user_message(message=text)
-
-                        # if user is typing, wait for typing to stop
-                        while True:
-                            if chat_with_participant.id not in self._is_typing:
-                                break
-
-                            await asyncio.sleep(0.5)
+                        chat_context.append_user_message(message=text)
 
                         if messages.empty():
                             break
@@ -606,17 +699,29 @@ class ChatBot(SingleRoomAgent):
                         span.set_attributes({"text": text})
 
                         try:
+                            for participant in get_thread_participants(
+                                room=self._room, thread=thread
+                            ):
+                                # TODO: async gather
+                                self._room.messaging.send_message_nowait(
+                                    to=participant,
+                                    type="thinking",
+                                    message={"thinking": True, "path": path},
+                                )
+
                             if thread_context is None:
                                 thread_context = ChatThreadContext(
+                                    path=path,
                                     chat=chat_context,
                                     thread=thread,
                                     participants=get_thread_participants(
                                         room=self.room, thread=thread
                                     ),
                                 )
-
-                            def handle_event(evt):
-                                llm_messages.send_nowait(evt)
+                            else:
+                                thread_context.participants = get_thread_participants(
+                                    room=self.room, thread=thread
+                                )
 
                             with tracer.start_as_current_span("chatbot.llm") as span:
                                 try:
@@ -630,6 +735,23 @@ class ChatBot(SingleRoomAgent):
                                             )
                                         )
 
+                                    await self.prepare_llm_context(
+                                        context=thread_context
+                                    )
+
+                                    llm_messages = asyncio.Queue[ResponseStreamEvent]()
+
+                                    def handle_event(evt):
+                                        llm_messages.put_nowait(evt)
+
+                                    llm_task = asyncio.create_task(
+                                        self._process_llm_events(
+                                            thread_context=thread_context,
+                                            llm_messages=llm_messages,
+                                            thread_attributes=thread_attributes,
+                                        )
+                                    )
+
                                     await self._llm_adapter.next(
                                         context=chat_context,
                                         room=self._room,
@@ -637,10 +759,14 @@ class ChatBot(SingleRoomAgent):
                                         tool_adapter=self._tool_adapter,
                                         event_handler=handle_event,
                                     )
+
+                                    llm_messages.shutdown()
+                                    await llm_task
+
                                 except Exception as e:
                                     logger.error("An error was encountered", exc_info=e)
                                     await self._send_and_save_chat(
-                                        messages=doc_messages,
+                                        thread=thread,
                                         to=chat_with_participant,
                                         path=path,
                                         id=str(uuid.uuid4()),
@@ -649,27 +775,30 @@ class ChatBot(SingleRoomAgent):
                                     )
 
                         finally:
-                            for participant in get_thread_participants(
-                                room=self._room, thread=thread
-                            ):
-                                # TODO: async gather
-                                self._room.messaging.send_message_nowait(
-                                    to=participant,
-                                    type="thinking",
-                                    message={"thinking": False, "path": path},
-                                )
+
+                            async def cleanup():
+                                for participant in get_thread_participants(
+                                    room=self._room, thread=thread
+                                ):
+                                    self._room.messaging.send_message_nowait(
+                                        to=participant,
+                                        type="thinking",
+                                        message={"thinking": False, "path": path},
+                                    )
+
+                            asyncio.shield(cleanup())
 
         finally:
-            llm_messages.close()
 
-            if self.room is not None:
-                logger.info("thread was ended {path}")
-                self.room.developer.log_nowait(
-                    type="chatbot.thread.ended", data={"path": path}
-                )
+            async def cleanup():
+                if self.room is not None:
+                    logger.info(f"thread was ended {path}")
+                    logger.info("chatbot thread ended", extra={"path": path})
 
-                if thread is not None:
-                    await self.close_thread(path=path)
+                    if thread is not None:
+                        await self.close_thread(path=path)
+
+            asyncio.shield(cleanup())
 
     def _get_message_channel(self, key: str) -> Chan[RoomMessage]:
         if key not in self._message_channels:
@@ -703,7 +832,7 @@ class ChatBot(SingleRoomAgent):
 
                 messages = self._get_message_channel(path)
 
-                logger.info(
+                logger.debug(
                     f"queued incoming message for thread {path}: {message.type}"
                 )
 
@@ -712,6 +841,7 @@ class ChatBot(SingleRoomAgent):
                 if path not in self._thread_tasks or self._thread_tasks[path].done():
 
                     def thread_done(task: asyncio.Task):
+                        self._thread_tasks.pop(path)
                         self._message_channels.pop(path)
                         try:
                             task.result()
@@ -722,7 +852,7 @@ class ChatBot(SingleRoomAgent):
                                 f"The chat thread ended with an error {e}", exc_info=e
                             )
 
-                    logger.info(f"spawning chat thread for {path}")
+                    logger.debug(f"spawning chat thread for {path}")
                     task = asyncio.create_task(
                         self._spawn_thread(messages=messages, path=path)
                     )
@@ -730,6 +860,11 @@ class ChatBot(SingleRoomAgent):
 
                     self._thread_tasks[path] = task
 
+            elif message.type == "cancel":
+                path = message.message["path"]
+                if path in self._thread_tasks:
+                    self._thread_tasks[path].cancel()
+
             elif message.type == "typing":
 
                 def callback(task: asyncio.Task):
